//*****************************************************************************
//*****************************************************************************
//  FILENAME:   sensor.c
//  Version: 1.0, 2007/03/08
//  Generated by Andy Kiser
//
//  DESCRIPTION:  sensor specific functions 
//
//-----------------------------------------------------------------------------
// Originally authored by Toradex AG. This work is now available under the terms and conditions
// of the Creative Commons License 'Attribution CC BY', details of which can be found here:
// http://creativecommons.org/licenses/by/3.0/
//*****************************************************************************
//*****************************************************************************
//#define __DEBUG__
#include <m8c.h>        // part specific constants and macros
#include "sensor.h"
#include "psocgpioint.h"
#include "feature.h"
#include "parameters.h"
#include "types_n_macros.h"
//----------------------------------------
// Sensor parameters
// Factory Defaults are defined here (NUM_xxx are defined in "sensor.h")
//----------------------------------------
const BYTE FactoryDefault_Param_8   [NUM_PARAM_8] =
{
	0x00,		// report mode
	0x00		// LED mode
};

const WORD FactoryDefault_Param_16  [NUM_PARAM_16] = 
{
	50,		// report rate [ms]
	50,		// sample rate [ms]
	50000,		// sample rate [us]	(not used for this sensor)
};

const DWORD FactoryDefault_Param_32  [NUM_PARAM_32] =
{
	0x00000000	// currently no 32bit parameters
};

const tParam_168 FactoryDefault_Param_168 [NUM_PARAM_168] =
{
	"Isolated Input",	// user device name
	"Frame Number",		// user name Channel 0
	"Inputs",			// user name Channel 1
};


//----------------------------------------
// this module's global variables
//----------------------------------------
tReport Report;

//----------------------------------------
// constant lookup-table for ADC->physical value conversion
//----------------------------------------


//----------------------------------------
// type definitions
//----------------------------------------

typedef enum 
{	
	eIdle
} tFSM_Measure;

//----------------------------------------
// variables

tFSM_Measure	FSM;
WORD			Countdown_1ms;

BOOL			NewSample = FALSE;

// store original drivemodes during suspend
BYTE			DM0_2, DM0_1, DM0_0;
BYTE			DM2_2, DM2_1, DM2_0;

//-----------------------------------------------------------------------------
// Initializes the Sensor 
//-----------------------------------------------------------------------------
void Sensor_Start ()
{
	FSM = eIdle;
	Countdown_1ms = 0;
	// turn on power for the optocouper inputs
	GPIO_HIGH(ON_nOFF)
}
//-----------------------------------------------------------------------------
// Prepare Sensor for Suspend
//-----------------------------------------------------------------------------
void Sensor_Suspend()
{
	// turn off power for the sensor
	GPIO_LOW(ON_nOFF)

	// store original drivemodes during  suspend
	DM0_2 = PRT0DM2;
	DM0_1 = PRT0DM1;
	DM0_0 = PRT0DM0;
	DM2_2 = PRT2DM2;
	DM2_1 = PRT2DM1;
	DM2_0 = PRT2DM0;
	
	// set all I/Os to/from the sensor to High-Z Analog (=110), 
	// except the OFF_nON signal. 
	// OFF_nON is on Port0, all other signals on Port2
	PRT0DM2 |= ~ON_nOFF_MASK;
	PRT0DM1 |= ~ON_nOFF_MASK;
	PRT0DM0 &=  ON_nOFF_MASK;
	PRT2DM2  = 0xFF;
	PRT2DM1  = 0xFF;
	PRT2DM0  = 0x00;

}
//-----------------------------------------------------------------------------
// Resume Sensor from Suspend mode
//-----------------------------------------------------------------------------
void Sensor_Resume()
{
	BYTE ReadMode;
	
	// restore drivemodes for GPIOs to/from the sensor
	PRT2DM2 = DM2_2;
	PRT2DM1 = DM2_1;
	PRT2DM0 = DM2_0;

	PRT0DM2 = DM0_2;
	PRT0DM1 = DM0_1;
	PRT0DM0 = DM0_0;

	// start up sensor again
	Sensor_Start();
}


//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// cannot be handled with the standard functionality in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//
//	return Value: number of Parameter bytes (1-31) that are 
//				  returned in the IN report
//-----------------------------------------------------------------------------
BYTE Sensor_Specific_Feature(void)
{
	// currently no special feature reports are handled
	return 0;
}

//-----------------------------------------------------------------------------
// Function that will be called when a FEATURE REPORT is received that 
// sets / gets a 8/16/32/168bit parameter in feature.c
// The feature report data is available in the "FEATURE_RPT" variable
//
// --> see feature.c for an example how to parse FEATURE_RPT
//-----------------------------------------------------------------------------
void Sensor_Specific_Parameter(BYTE *NumberOfReturnedBytes)
{
	// currently there are no specific parameters
}

//-----------------------------------------------------------------------------
// void Sensor_FSM_Measure(void)
//
//	finite state machine to control the measurement process for 
//  the Sensor
//
//  IMPORTANT: 	call this routine once every millisecond. Otherwise the timing
//				parameters cannot be met.
//-----------------------------------------------------------------------------
void Sensor_FSM_Measure() 
{

	// decrement millisecond counter
	if (Countdown_1ms != 0)
		Countdown_1ms--;

	// process actual state machine
	switch (FSM)
	{
		case eIdle:
			if (Countdown_1ms == 0)
			{
				Get_Param_16(PAR16_SAMPLERATE_MS, VOLATILE, &Countdown_1ms);
				Report.OIN = ~OIN_1_Data_ADDR;
				NewSample = TRUE;
			}
			break;			
	}
}

//-----------------------------------------------------------------------------
void Sensor_Debug(void)
{
/* currently no debug function */
}